/**
 * @hide
 * Documents a function.
 * Doc can guess at a functions name and params if the source following a comment
 * matches something like:
 * @code_start
 * myFuncOne : function(param1, param2){}  //or
 * myFuncTwo = function(param1, param2){} 
 * @code_end
 * <h3>Directives</h3>
 * Use the following directives to document a function.
<pre>
[MVC.Doc.Function|@function] <i>function_name</i>                       -> Forces a function
[MVC.Doc.Directive.Param|@param] {<i>optional:type</i>} <i>param_name</i> <i>Description</i> -> Describes a parameter
[MVC.Doc.Directive.Return|@return] {<i>type</i>} <i>Description</i>                    -> Describes the return value
</pre>
 * Add <i>optional:</i> for optional params.  Other available directives: 
 * [MVC.Doc.Directive.Plugin|@plugin],[MVC.Doc.Directive.CodeStart|@code_start]
 * <h3>Example</h3>
@code_start
/* Adds, Mr. or Ms. before someone's name
* * @param {String} name the persons name
* * @param {optional:Boolean} gender true if a man, false if female.  Defaults to true.
* * @return {String} returns the appropriate honorific before the person's name.
* *| 
 honorific = function(name, gender){
@code_end
 */
MVC.Doc.Function = MVC.Doc.Pair.extend('function',
/* @static */
{
    code_match: /(?:([\w\.]+)|(["'][^"']+["']))\s*[:=]\s*function\(([^\)]*)/,
    init : function(){
        this.add(MVC.Doc.Directive.Return, MVC.Doc.Directive.Param, MVC.Doc.Directive.CodeStart, 
        MVC.Doc.Directive.CodeEnd,MVC.Doc.Directive.Plugin)
        this._super();
    }
},
/* @prototype */
{
    code_setup: function(){
        var parts = this.Class.code_match(this.code);

        if(!parts){
            parts = this.code.match(/\s*function\s+([\w\.\$]+)\s*(~)?\(([^\)]*)/)
        }
        
        this.name = parts[1] ? parts[1].replace(/^this\./,"") : parts[2];
        this.params = {};
        this.ret = {type: 'undefined',description: ""}
        var params = parts[3].match(/\w+/g);

        if(!params) return;
        
        for(var i = 0 ; i < params.length; i++){
            this.params[params[i]] = {description: "", type: "", optional: false, order: i, name: params[i]};
        }
    },
    

    /**
     * Sets the function's name if one can't be determined from the source
     * @param {Object} line
     */
    function_add: function(line){
        var m = line.match(/^@\w+\s+([\w\.\$]+)/)
        if(m) this.name = m[1];
    },
    /**
     * Returns the HTML signiture of the function.
     */
    signiture : function(){
        var res = [];
        var ordered = this.ordered_params();
        for(var n = 0; n < ordered.length; n++){
            res.push(ordered[n].name)
        }
        
        var n = this.name;
        return n+"("+res.join(", ")+") -> "+this.ret.type;
    }
});